"""
This files contains kind of a "wrapper function" for Airtable API and utility functions.
"""

import json
import requests
from requests.packages.urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter


# Basic logging
def log(*args):
    for thing in args:
        if type(thing) is dict:
            thing = json.dumps(thing)
        print('Airtable plugin - %s' % thing)


# Session object for requests
s = requests.Session()
# Retry strategy (cf http://stackoverflow.com/a/35504626/4969056)
retries = Retry(total=3,
                backoff_factor=2)
s.mount('https://', HTTPAdapter(max_retries=retries))


def airtable_api(base, table, token, action='', parameters={}, method='get', data={}):
    """
    Helper function to make calls to Airtable REST API.
    """

    headers = {
        'Content-type': 'application/json',
        'Accept-Encoding': 'gzip',
        'Authorization': 'Bearer %s' % token
    }
    url = "https://api.airtable.com/v0/%s/%s/%s" % (base, table, action)
    if method == 'get':
        r = s.request(method, url, headers=headers, params=parameters, timeout=10)
    elif method == 'post':
        r = s.request(method, url, headers=headers, data=json.dumps(data), params=parameters, timeout=10)
    else:
        raise Exception('Method should be get or post.')
    log('API %s call: %s' % (method, r.url))
    if r.status_code < 300:
        return r.json()
    else:
        error_message = get_error_message(r)
        raise Exception(error_message)


def safe_airtable_api(base, table, token, action='', parameters={}, method='get', data={}):
    try:
        response = airtable_api(base, table, token, action=action, parameters=parameters, method=method, data=data)
    except Exception as error:
        return None, "{}".format(error)
    return response, None


def get_error_message(response):
    error_message = 'API error ({}) : {}'.format(response.status_code, response.content)
    log(error_message)
    try:
        json_content = json.loads(response.content)
    except Exception:
        return error_message
    enriched_error_message = json_content.get("error", {}).get("message", None)
    if enriched_error_message:
        error_message = "{} - (Error {})".format(enriched_error_message, response.status_code)
    return error_message
